// AppController.m -- application controller, handler of all the buttons.

// Copyright (C) 2003 Borkware
// We use the BSD License.  Check out http://borkware.com/license


#import "AppController.h"

@implementation AppController

// no init method, since we have no initialization to do


// clean up our mess

- (void) dealloc
{
    [sound release];
    [super dealloc];

} // dealloc



// walk the /System/Library/Sounds directory and pick up any
// files that NSSound will grok.  Stick them into a pop-up menu
// and let the user choose amongst them

- (void) populateSoundNameMenu
{
    // clean out any junk left by Interface Builder
    [soundNamesPopup removeAllItems];

    // pick up stuff in /System/Library/Sounds

    NSArray *dirContents;
    dirContents = [[NSFileManager defaultManager]
		      directoryContentsAtPath: @"/System/Library/Sounds"];

    NSEnumerator *enumerator;
    enumerator = [dirContents objectEnumerator];
    
    NSString *filename;
    while (filename = [enumerator nextObject]) {

	// test the extension against NSSound's file types array
	NSString *extension;
	extension = [filename pathExtension];

	int index;
	index = [[NSSound soundUnfilteredFileTypes]
		    indexOfObject: extension];

	// it's a match!  add just the title (without the extension)
	// to the menu
	if (index != NSNotFound) {
	    [soundNamesPopup 
		addItemWithTitle: [filename stringByDeletingPathExtension]];
	}
    }
    
} // populateSoundNameMenu



// our niblets have loaded.  fill in the System Sound popup menu
// with the system sounds

- (void) awakeFromNib
{
    [self populateSoundNameMenu];
} // awakeFromNib



// button action handler.  Load the named sound the user has selected

- (IBAction) loadNamedSound: (id) sender
{
    // pick up the name from the menu
    NSString *name;
    name = [soundNamesPopup titleOfSelectedItem];

    // get rid of any currently loaded sound
    [sound release];

    // get the new sound
    sound = [NSSound soundNamed: name];

    // enable/disable the button based on success of loading
    [playButton setEnabled: (sound != nil)];

} // loadNamedSound



// button action handler.  Load a file that has a sound that NSSound can
// consume

- (IBAction) loadSoundFromFile: (id) sender
{
    // use an open panel file chooser modal sheeting thingie

    NSOpenPanel *fileChooser;
    fileChooser = [NSOpenPanel openPanel];

    // just load one file at a time.  and no directories!

    [fileChooser setAllowsMultipleSelection: NO];
    [fileChooser setCanChooseDirectories: NO];

    // kick off the sheet

    [fileChooser beginSheetForDirectory: @""
		 file: @""
		 types: [NSSound soundUnfilteredFileTypes]
		 modalForWindow: window
		 modalDelegate: self
		 didEndSelector: @selector(openPanelDidEnd:returnCode:context:)
		 contextInfo: nil];

} // loadSoundFromFile



// sheet delegate handler.  This gets invoked by the file chooser open file
// thingie when the user selects an item, or cancels it

- (void) openPanelDidEnd: (NSOpenPanel *) chooser
	      returnCode: (int) returnCode
		 context: (id) context
{
    // only look at stuff on an OK click

    if (returnCode == NSOKButton) {

	// get the item selected
	NSString *filename;
	filename = [[chooser filenames] objectAtIndex: 0];

	// get rid of any currently loaded sound
	[sound release];

	// get the new sound
	sound = [[NSSound alloc] 
		    initWithContentsOfFile: filename
		    byReference: NO];
	// (the by reference is really only used when using
	// encodeWithCoder: on the sound)

	// enable/disable the button based on success of loading
	[playButton setEnabled: (sound != nil)];
    }

} // openPanelDidEnd



// button action handler.  Load an URL that has a sound.  We default
// to an url at Borkware that has a snippet of the haunting Torgo Theme
// from '"Manos" the Hands of Fate'

- (IBAction) loadSoundFromURL: (id) sender
{
    // create an url with our locatio
    NSURL *youllUrl;
    youllUrl = [NSURL URLWithString:
			  @"http://borkware.com/rants/sound/torgo-short.aiff"];

    // get rid of any currently loaded sound
    [sound release];

    // get the new sound
    sound = [[NSSound alloc]
		initWithContentsOfURL: youllUrl
		byReference: NO];

    // enable/disable the button based on success of loading
    [playButton setEnabled: (sound != nil)];

} // loadSoundFromURL


- (IBAction) play: (id) sender
{
    // start the sound playing.  sounds from URLs might take a couple of
    // attempts to start playing
    [sound play];

} // play


@end // AppController


